option explicit

' ****************************************************************************
' *** VERSION ****************************************************************
' ****************************************************************************
'
' VMware_Launcher.vbs
'   v1.00
'   12/Aug/2009 (updated for VMware 7 05/Feb/2010)
'   by Leo Davidson <leo@ox.compsoc.net> http://www.pretentiousname.com
'   Tested on Windows 7 RTM x64 with VMware Workstaiton 6.5.2 and 7.0.0

' ****************************************************************************
' *** PURPOSE ****************************************************************
' ****************************************************************************
'
' VMware is a great tool but, sadly, it runs up to four services at startup
' when those services are not needed while VMware is not in use and, depending
' on configuration, may not be needed at all. These extra services needlessly
' slow down your machine's boot and at least one of them is constantly
' polling the filesystem and/or registry (instead of waiting on a change
' notification event and going to sleep like better-written code would do).
' That constant activity means that Windows cannot page the process to disk.
'
' (There is also the vmware-tray.exe process which constantly polls an ini
' file, even if you are not running VMware and have disabled its tray icon.
' Stopping that is easy and can be done using a tool like AutoRuns. Nothing
' at all depends on that stupid process so get rid of it if you don't use it.)
'
' This script, along with the instructions in the PERMISSIONS section below,
' allows you to automatically start the required VMware service(s) when
' you launch VMware and stop the service(s) when you exit VMware. In other
' words, this script does what VMware itself should do and what people have
' been complaining about since at least 2007. :-)
'
' It's up to you to manually configure the four VMware services to "Manual"
' start (instead of "Automatic") so that they don't run at startup. You can
' do that using the Services control panel, under Administrator Tools in
' the Start Menu or Control Panel. (Where it is depends on your Start Menu
' configuration.)
'
' Having configured the services not to auto-start, you should then follow
' the instructions in the PERMISSIONS section, and from then on launch
' VMware using this script instead of directly. You may also need to
' edit some of the paths, or the list of serivces to start & stop, in the
' CONFIG section below.

' ****************************************************************************
' *** CONFIG *****************************************************************
' ****************************************************************************

' Change this to where VMware is installed. If you're using 32-bit Windows, or
' a 64-bit version of VMware is released, you'll probably need to remove
' the " (x86)" from the path.

Dim VMwareHome
VMwareHome = "C:\Program Files (x86)\VMware\VMware Workstation"

Dim VMwareExe
VMwareExe = "vmware.exe"

' Only include in here the services you need to start/stop. For me it's just the
' Authorization and USB Arbitration services but you may also require some of the
' others, depending on your config.
' Note that the "VMware USB Arbitration Service" was added in VMware 7; if you are
' using VMware 6 then it won't exist on your system and you should remove/ignore it.

Dim VMwareServices
VMwareServices = Array("VMware Authorization Service", "VMware USB Arbitration Service")
' VMwareServices = Array("VMware Authorization Service")
' VMwareServices = Array("VMware Authorization Service", "VMware Agent Service", "VMware USB Arbitration Service")
' VMwareServices = Array("VMware Authorization Service", "VMware Agent Service", "VMware USB Arbitration Service", "VMware DHCP Service", "VMware NAT Service")

' ****************************************************************************
' *** PERMISSIONS ************************************************************
' ****************************************************************************
'
' If you are running this from a non-admin account (including an admin account
' under UAC) then you will need to grant your account permission to start and
' stop the VMware services. Since the services would normally be running 24/7
' anyway this should not pose a security risk, provided you are the only
' VMware user on the machine. (This script is absolutely not safe if there are
' multiple VMware users as it will stop the services that other people still
' depend on.)
'
' The easiest way to grant yourself permission is to use the SubInAcl.exe tool
' which you can download from Microsoft:
'
' http://www.microsoft.com/downloads/details.aspx?FamilyId=E8BA3E56-D8FE-4A91-93CF-ED6985E3927B&displaylang=en 
'
' Note that while the webpage says "Windows 2000; Windows Server 2003; Windows XP"
' the tool also works fine (at least for me!) on Windows 7 and presumably Windows
' Vista and Windows Server 2008.
'
' Once you have SubInAcl downloaded and installed, open an Administrator
' command prompt and CD to where the tool was installed. On 64-bit Windows it's:
'
' C:\Program Files (x86)\Windows Resource Kits\Tools
'
' On 32-bit Windows you should remove the " (x86)" part.
'
' You need to grant your account permission to start & stop the services you
' need. Which VMware services you need depends on your configuration. I only
' need the VMWare Agent Service but you might need others.
'
' IMPORTANT: Do not mess around with SubInAcl unless you're confident &
' competent at performing admin/permissioning tasks. You can completely mess
' up your machine if you do something stupid. Add "/testmode" as the first
' argument to SubInAcl.exe if you are unsure; it will simulate the command
' and show you the result without making any changes for real.
'
' SubInAcl.exe requires the low-level service name, not the "display name"
' shown in Control Panel and used in the lists above. Here are the low-level
' names which correspond to each of the display names. NOTE that these
' service names may change with different versions of VMware. You can use
' a tool such as AutoRuns to get a list of the low-level names. (Launch
' AutoRuns, go to its Services tab and then sort by Publisher to quickly
' find all the VMware services.)
'
' "ufad-ws60"          is the low-level name for "VMware Agent Service"
' "VMAuthdService"     is the low-level name for "VMware Authorization Service"
' "VMnetDHCP"          is the low-level name for "VMware DHCP Service"
' "VMware NAT Service" is the low-level name for "VMware NAT Service"
' "VMUSBArbService"    is the low-level name for "VMware USB Arbitration Service"
'
' Note that the "VMware USB Arbitration Service" was added in VMware 7; if you are
' using VMware 6 then it won't exist on your system and you should remove/ignore it.
'
' To give an account called Robert Paulson permission to start and stop
' VMAuthdService run this:
'
' subinacl.exe /service "VMAuthdService" /grant="Robert Paulson"=TO /display
'
' The "=TO" part means you're granting that account sTart and stOp access.
'
' If/when adding permissions for the "VMware NAT Service" serivce,
' you *must* put quotes around the service name in the command-line!
' For the other serivices, which don't have a space in the name, the quotes
' are optional.
'
' The account name is the actual name of the account, not the "Full Name".
' It should be the same as the folder created for the account below C:\Users
' or C:\Documents and Settings. If the account name contains a space
' then you need to put quotes around it, like in the example. If not then
' the quotes are optional.
'
' As I said above, you can use /testmode to simulate the change without
' actually making it:
'
' subinacl.exe /testmode /service "VMAuthdService" /grant="Robert Paulson"=TO /display
'
' Don't forget to permission each of the services you need, not just VMAuthdService.

' ****************************************************************************
' *** UTILS ******************************************************************
' ****************************************************************************

Private Function ProcessIsRunning( wmi, strProcess )
	Dim ProcessList

	Set ProcessList = wmi.Execquery("Select * from Win32_Process Where Name ='" & strProcess & "'")

	If ProcessList.Count > 0 Then
		ProcessIsRunning = True
	Else
		ProcessIsRunning = False
	End If
End Function

Private Function ControlService( wmiServices, ServiceName, boolStart )
	Dim ServiceList
	Dim Service
	Dim Res
	Dim ActionName

	Set ServiceList = wmiServices.ExecQuery("Select * from Win32_Service where DisplayName='" & ServiceName & "'")

	If ServiceList.Count = 0 Then
		MsgBox Wscript.ScriptName & ": Service with display name """ & ServiceName & """ could not be found."
		WScript.Quit 1
	End If

	If ServiceList.Count > 1 Then
		MsgBox Wscript.ScriptName & ": Multiple services with display name """ & ServiceName & """ were found."
		WScript.Quit 1
	End If

	For Each Service In ServiceList
		If (boolStart) Then
			Res = Service.StartService
			ActionName = "start"
		Else
			Res = Service.StopService
			ActionName = "stop"
		End If

		' 0 = Success; 10 = Service Already Running/Stopped
		If (Res <> 0 And Res <> 10) Then
			If (Res = 2) Then
				MsgBox Wscript.ScriptName & ": Service """ & ServiceName & """ failed to " & ActionName & " (Access Denied). You probably need to use SubInAcl to give your account permission to start/stop the service. See the comments at the top of the script."
			Else
				MsgBox Wscript.ScriptName & ": Service """ & ServiceName & """ failed to " & ActionName & " (" & Res & ")"
			End If
			If (boolStart) Then
				WScript.Quit 1
			End If
		End If
	Next
End Function

' ****************************************************************************
' *** MAIN *******************************************************************
' ****************************************************************************

Dim Shell
Set Shell = CreateObject("WScript.Shell")

Dim wmi
Set wmi = Getobject("winmgmts:")

Dim wmiServices
Set wmiServices = GetObject("winmgmts:\\.\root\cimv2")

Dim ServiceName

' ----------------------------------------------------------------------------
' If VMware.exe is already running then we'll quit with an error message
' because we don't want to be starting & stopping VMware services while
' another copy of VMware, and possibly another copy of this script.
' ----------------------------------------------------------------------------

If ProcessIsRunning( wmi, VMwareExe ) Then
	MsgBox WScript.ScriptName & ": VMware is already running."
	WScript.Quit 1
End If

' ----------------------------------------------------------------------------
' Start the service(s)
' ----------------------------------------------------------------------------

For Each ServiceName In VMwareServices
	ControlService wmiServices, ServiceName, True
Next

' ----------------------------------------------------------------------------
' Launch VMware.exe and wait for it to exit.
' ----------------------------------------------------------------------------

Shell.Run """" & VMwareHome & "\" & VMwareExe & """", 1, True

' ----------------------------------------------------------------------------
' Stop the service(s)
' ----------------------------------------------------------------------------

For Each ServiceName In VMwareServices
	ControlService wmiServices, ServiceName, False
Next
